import org.apache.spark.rdd.RDD
import breeze.interpolation._
import breeze.linalg._
import java.io._


// Directorio para los datos
val PATH = "/user/WDN/rawData/senso1/"
val DATA_SERIE = "/*"
val ATTR_HOUR = 0
val ATTR_FORESCAST = 1
val ATTR_MEASURED = 2


// Creamos la lista de archivos que contiene el Directorio
def getListOfFiles(path: String):List[File] = {
val d = new File(path)
if (d.exists && d.isDirectory) {
d.listFiles.filter(_.isFile).toList
} else {
List[File]()
}
}
/* Función que lee cada archivo y los transforma a RDD */
def readData(dir: String): (RDD[Array[String]]) = {
// Leemos datos y creamos RDD de String
var data_serie:RDD[String] = sc.textFile(PATH + dir + DATA_SERIE)

// Quitamos el encabezado
var serie_filtered:RDD[String] = data_serie.filter(line => !line.contains("Category"))

//Dividimos por comas
var data:RDD[Array[String]] =  serie_filtered.map(line => line.split (","))
println(PATH + dir + DATA_SERIE)  
(data)
}

def interpolation(data: RDD[Array[String]],dir: String): Unit = {
// Extraemos cada atributo para obtener la serie
var serie_hour:RDD[String]=data.map{a => a(ATTR_HOUR)}
var serie_measured:RDD[String]=data.map{a => a(ATTR_MEASURED)}

// Quitamos los valores nulos de la serie (si los hay) y creamos el DenseVector "y" 
var vector:RDD[Double] = serie_measured.filter(n => !n.contains("null")).map{ x => x.toDouble }
var y = new DenseVector(vector.collect)

// Para crear el DenseVector "x" obtenemos las posiciones de los valores no nulos, con ayuda de index
var index:RDD[Double] = serie_measured.zipWithIndex.filter((ind: (String, Long)) => ind._1 != "null").map{case (k,v) => (v).toDouble}
var x = new DenseVector(index.collect)

// Usamos la funcion "LinearInterpolator" de la biblioteca BREEZE
var interpolated = LinearInterpolator(x, y)

// Un Time series completo tiene 144 valores, por eso solicitamos la interpolación para un DenseVector 0-143
var ts = DenseVector.range(0,144,1)
var out = interpolated(ts)

// creamos un Map con el valor inicial hour y el resultado de la interpolación
var serie_out = (serie_hour.toArray zip out.toArray).toMap
sc.parallelize(serie_out.toSeq).saveAsTextFile("hdfs://quickstart.cloudera:8019/user/WDN/T0/"+dir)

}

/***************************************************
******************* LECTURA DATOS ******************
****************************************************/
val Folder = getListOfFiles(PATH)

for (i <- 0.until(Folder.length)) {
	var dir = Folder(i)
	data = readData(dir)

	data.map{a => 
	if(a(ATTR_MEASURED).contains("null")){
		interpolation(data,dir)
		println("contiene null")
	}
	else{
		println("NO contiene null")
		var hour=a(ATTR_HOUR)
		var measured = a(ATTR_MEASURED).toDouble
		var serie_out = (hour.toArray zip measured.toArray).toMap
		sc.parallelize(serie_out.toSeq).saveAsTextFile("hdfs://quickstart.cloudera:8019/user/WDN/T0/"+dir)
	}
	}
}